
#include "VKSimData.hh"

#include <TSystem.h>
#include <TApplication.h>
#include <TParameter.h>

#include <stdexcept>    // std::invalid_argument

using namespace std;


VKSimData::VKSimData(string filename)
{
    // get tree
    string errMessage = filename + " does not exist.";
    if (gSystem->AccessPathName(filename.c_str())) throw invalid_argument(errMessage);
    fInputFile  = new TFile(filename.c_str(), "READ");
    fInputTree = (TTree*)fInputFile -> Get("Detector"); 
    SetInputBranches(fInputTree);

    // number of events going to output file
    fEntries = fInputTree -> GetEntries();

    // normalization
    fNormalization = 
        ((TParameter<double>*)(fInputTree->GetUserInfo()->FindObject("NORMALIZATION")))->GetVal();

    // area
    fArea =
        ((TParameter<double>*)(fInputTree->GetUserInfo()->FindObject("SOURCE_AREA/cm2")))->GetVal();
}


VKSimData::~VKSimData() 
{
    // make sure input ROOT file has been closed
    if (fInputFile != NULL && fInputFile->IsOpen()) fInputFile->Close();

    // delete pointers
    delete fInputFile;
    delete fInputTree;
}


void VKSimData::SetInputBranches(TTree * t) 
{
    t -> SetBranchAddress("EDepSignal", &_ESin);
    t -> SetBranchAddress("EDepVeto"  , &_EVin);
    t -> SetBranchAddress("PhotonsSignal", &_PSin);
    t -> SetBranchAddress("PhotonsVeto"  , &_PVin);
}


const VKEvent& VKSimData::GetEvent(size_t id)
{
    fInputTree -> GetEvent(id);
    fEvent.ES = _ESin;
    fEvent.EV = _EVin;
    fEvent.PS = _PSin;
    fEvent.PV = _PVin;
    return fEvent;
}
