//
// ********************************************************************
// * License and Disclaimer                                           *
// *                                                                  *
// * The  Geant4 software  is  copyright of the Copyright Holders  of *
// * the Geant4 Collaboration.  It is provided  under  the terms  and *
// * conditions of the Geant4 Software License,  included in the file *
// * LICENSE and available at  http://cern.ch/geant4/license .  These *
// * include a list of copyright holders.                             *
// *                                                                  *
// * Neither the authors of this software system, nor their employing *
// * institutes,nor the agencies providing financial support for this *
// * work  make  any representation or  warranty, express or implied, *
// * regarding  this  software system or assume any liability for its *
// * use.  Please see the license in the file  LICENSE  and URL above *
// * for the full disclaimer and the limitation of liability.         *
// *                                                                  *
// * This  code  implementation is the result of  the  scientific and *
// * technical work of the GEANT4 collaboration.                      *
// * By using,  copying,  modifying or  distributing the software (or *
// * any work based  on the software)  you  agree  to acknowledge its *
// * use  in  resulting  scientific  publications,  and indicate your *
// * acceptance of all terms of the Geant4 Software license.          *
// ********************************************************************
//
#include "VKPhysicsList.hh"

#include "VKGeneralPhysics.hh"
#include "VKEMPhysics.hh"
#include "VKMuonPhysics.hh"
#include "VKPhysicsListMessenger.hh"

#include "G4EmLowEPPhysics.hh"
#include "G4EmLivermorePhysics.hh"
#include "G4OpticalPhysics.hh"
#include "G4OpticalProcessIndex.hh"

#include "G4RunManager.hh"
#include "G4SystemOfUnits.hh"

#include "QGSP_BIC_HP.hh"


VKPhysicsList::VKPhysicsList() 
 : G4VModularPhysicsList(),
   fOpticalPhysicsOn(true)
{
    // create messenger class
    fPhysicsList_msg = new VKPhysicsListMessenger(this);
    
    // default cut value  (1.0mm)
    defaultCutValue = 1.0*mm;

    // General Physics
    RegisterPhysics(new VKGeneralPhysics("general"));

    // EM Physics
    RegisterPhysics(new VKEMPhysics("standard EM"));
    // RegisterPhysics(new G4EmStandardPhysics_option4()); // not yet tested
    // RegisterPhysics(new G4EmLivermorePhysics());  // overpredicts muon energy loss by a factor of 2!
    // RegisterPhysics(new G4EmLowEPPhysics());
    // RegisterPhysics(new G4EmPenelopePhysics()); // not yet tested
    
    // Muon Physics
    RegisterPhysics(new VKMuonPhysics("muon"));

    // Register optical Physics
    if (fOpticalPhysicsOn) {
        G4cout << " ---> Optical physics enabled by default" << G4endl;
        RegisterOpticalPhysics();
    }
    
//***    // Neutron and ion physics
//***    RegisterPhysics(new G4HadronPhysicsQGSP_BIC_HP());
//***    RegisterPhysics(new G4IonPhysics());
}


VKPhysicsList::~VKPhysicsList() 
{
    // do nothing
}


void VKPhysicsList::SetCuts(){
  //  " G4VUserPhysicsList::SetCutsWithDefault" method sets
  //   the default cut value for all particle types
  SetCutsWithDefault();
}


void VKPhysicsList::RegisterOpticalPhysics()
{
    // optical physics constructor
    G4OpticalPhysics * opticalPhysics = new G4OpticalPhysics();
    RegisterPhysics( opticalPhysics );
    
    // adjust optical physics
    opticalPhysics -> SetWLSTimeProfile("delta");
    opticalPhysics -> SetScintillationYieldFactor(1.0);
    opticalPhysics -> SetScintillationExcitationRatio(0.0);
    opticalPhysics -> SetMaxNumPhotonsPerStep(100);
    opticalPhysics -> SetMaxBetaChangePerStep(10.0);
    opticalPhysics -> SetTrackSecondariesFirst(kCerenkov,true);
    opticalPhysics -> SetTrackSecondariesFirst(kScintillation,true);
    
    // store name
    fOpticalPhysicsName = opticalPhysics -> GetPhysicsName();
}


void VKPhysicsList::SetOpticalPhysicsOn(G4bool b)
{
    if (fOpticalPhysicsOn && !b) {
        G4cout << " ---> Disabling optical physics" << G4endl;
        RemovePhysics (fOpticalPhysicsName);
        G4RunManager::GetRunManager() -> PhysicsHasBeenModified();
    } 
    else if (!fOpticalPhysicsOn && b) {
        G4cout << " ---> Enabling optical physics" << G4endl;
        RegisterOpticalPhysics();
        G4RunManager::GetRunManager() -> PhysicsHasBeenModified();
    } 
    else {
        // do nothing
    }
    fOpticalPhysicsOn = b;
}

